#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min','max';
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'xy_rotate_plus90','xy_rotate_minus90';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

# 10 x-n y
# 10 x-n y-n
my ($level, @coordinate_list) = (10, 'x-n', 'y');
if (@ARGV) { ($level, @coordinate_list) = @ARGV; }

print "  % generated by tools/$FindBin::Script $level ",
  join(' ',@coordinate_list),"\n";

my $path = Math::PlanePath::AlternatePaper->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $scale = 2**($level/2);
my $scale2 = $scale*$scale;

sub coord_to_str {
  my ($x) = @_;
  $x = sprintf '%.6f', $x/$scale2;
  $x =~ s/\.?0*$//;       # trailing .000 or 1000
  $x =~ s/^0\././;        # leading 0.
  return $x;
}

foreach my $i (0 .. $#coordinate_list) {
  my $coordinate = $coordinate_list[$i];
  my $colour = ($i == 0 ? '' : '[my grey] ');
  my ($x,$y) = $path->n_to_xy($n_lo);
  my @values_min;
  my @values_max;
  foreach my $n ($n_lo .. $n_hi-1) {
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my ($x2,$y2) = ($x+$dx, $y+$dy);
    my ($x3,$y3);
    if ($n&1) {
      # odd, reverse
      my ($lx,$ly) = xy_rotate_minus90($dx,$dy);
      ($x3,$y3) = ($x+$lx, $y+$ly);
    } else {
      # even, forward
      my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
      ($x3,$y3) = ($x2+$lx, $y2+$ly);
    }

    my $ns = $n/$scale;
    my @value_list
      = ($coordinate   eq 'x'   ? ($x*$scale,   $x2*$scale,   $x3*$scale)
         : $coordinate eq 'y'   ? ($y*$scale,   $y2*$scale,   $y3*$scale)
         : $coordinate eq 'x-n' ? ($x*$scale-$n,$x2*$scale-$n,$x3*$scale-$n,
                                   $x*$scale-($n+1),$x2*$scale-($n+1),$x3*$scale-($n+1))
         : $coordinate eq 'y-n' ? ($y*$scale-$n,$y2*$scale-$n,$y3*$scale-$n,
                                   $y*$scale-($n+1),$y2*$scale-($n+1),$y3*$scale-($n+1))
         : die);

    ### at: "n=$n  $x,$y  $x2,$y2  $x3,$y3  list ".join(',',@value_list)

    $values_min[$n] = min(@value_list);
    $values_max[$n] = max(@value_list);
    ($x,$y) = ($x2,$y2);
  }

  ### @values_min
  ### @values_max

  # foreach my $n (0 .. $#values_min-1) {
  #   if ($values_min[$n] > $values_max[$n+1]
  #       || $values_max[$n] < $values_min[$n+1]) {
  #     print "disjoint blocks: n=$n  $values_min[$n] > $values_max[$n+1]   $values_max[$n] < $values_min[$n+1]";
  #     die;
  #   }
  # }

  my $segments = MyLineSegments->new;
  $segments->add($n_lo,$values_min[$n_lo],       # start vertical
                 $n_lo,$values_max[$n_lo]);
  $segments->add($n_hi,$values_min[-1],        # end vertical
                 $n_hi,$values_max[-1]);

  foreach my $n (0 .. $#values_min) {
    foreach my $value ($values_min[$n], $values_max[$n]) {
      $segments->add($n,$value, $n+1,$value);    # horizontal
    }
    if ($n > $n_lo) {
      $segments->add($n,$values_min[$n-1],   # vertical sides
                     $n,$values_min[$n]);
      $segments->add($n,$values_max[$n-1],
                     $n,$values_max[$n]);

      if ($values_max[$n-1] < $values_min[$n]) {
        $segments->add($n,$values_max[$n-1],   # vertical joiner
                       $n,$values_min[$n]);
      }

      if ($values_min[$n-1] > $values_max[$n]) {
        $segments->add($n,$values_min[$n-1],   # vertical joiner
                       $n,$values_max[$n]);
      }
    }
  }

  $Printwrap = 2;
  printwrap("\\fill$colour ");
  $Printwrap += 2;
  $segments->printwrap (stringize => sub {
                          my ($x,$y) = @_;
                          $x = coord_to_str($x);
                          $y = coord_to_str($y);
                          return "($x,$y)";
                        });
  printwrap(";");
  my $count = $segments->lines_joined_count_points;
  printwrap(" % using $count points for n_hi=$n_hi\n");
}

print "  % end generated\n";
exit 0;
